
# Core Python imports.
import ctypes
import datetime
import os
import platform
import socket

def GetPrimaryIp(target_address = "8.8.8.8", target_port = 80):
    """ Get the system default IP address.

    This is not as simple as I expected it to be...

    On Windows you can find this rather easily using the socket module:
    hostname = socket.gethostname()
    ip = socket.gethostbyname(hostname)

    However, on the Ubuntu box I has at the time, this returned 127.0.1.1.

    After a little Googling I stumbled upon this solution:
    https://stackoverflow.com/questions/166506/finding-local-ip-addresses-using-pythons-stdlib

    My understanding of this answer is we create a socket and ask the
    operating system to connect it to some arbitrary internet location. The
    operating system will then use the system's default IP address to make the
    connection. Once the connection is finished, regardless of whether it has
    succeeded or failed, the OS will have chosen a sensible IP address which
    we can use.

    I have provided default address and port values, you may wish to override
    these if you're trying to select a specific IP address on a machine with
    multiple NICs.
    """

    # Create the socket and attempt to connect to the target.
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.connect((target_address, target_port))

    # Get the socket name. This takes the form (ip, port) so we just return
    # the first component, the IP address.
    ip = sock.getsockname()[0]

    # Close the socket and return the IP.
    sock.close()
    return ip

def GetUptime():
    """ Get machine uptime in seconds.
    """

    # Windows.
    if platform.system() == "Windows":

        # Get library with function we need.
        lib = ctypes.windll.kernel32

        # Call function that gives us uptime in milliseconds.
        ut_millis = lib.GetTickCount64()

        # Return as seconds.
        return ut_millis / 1000

    # Linux.
    elif platform.system() == "Linux":

        # Use uptime to get time this machine started.
        started_str = os.popen("uptime -s").read()[:-1]
        started_dt = datetime.datetime.strptime(started_str, "%Y-%m-%d %H:%M:%S")

        # Find difference between start time and time now.
        time_now = datetime.datetime.now()
        time_diff = time_now - started_dt

        # Return time difference in seconds.
        return time_diff.total_seconds()

    # Un-supported platforms.
    else:
        print("ERROR: GetUptime() unsupported platform")
        return 0




class CActiveObject:
    def __init__(self, name):
        self.Name = name


        self.MsgsIn  = []
        self.MsgsOut = []

        self.Running = False

    def Start(self):
        self.Running = True

    def RxMsg(self, msg):
        self.MsgsIn.append(msg)










"""
Colour printing in vanilla Python.
"""

from enum import Enum

class _EAnsiColours(Enum):
    FGBLACK   = "30"
    FGRED     = "31"
    FGGREEN   = "32"
    FGYELLOW  = "33"
    FGBLUE    = "34"
    FGMAGENTA = "35"
    FGCYAN    = "36"
    FGWHITE   = "37"

    BGBLACK   = "40"
    BGRED     = "41"
    BGGREEN   = "42"
    BGYELLOW  = "43"
    BGBLUE    = "44"
    BGMAGENTA = "45"
    BGCYAN    = "46"
    BGWHITE   = "47"

    FGBRIGHTBLACK   = "90"
    FGBRIGHTRED     = "91"
    FGBRIGHTGREEN   = "92"
    FGBRIGHTYELLOW  = "93"
    FGBRIGHTBLUE    = "94"
    FGBRIGHTMAGENTA = "95"
    FGBRIGHTCYAN    = "96"
    FGBRIGHTWHITE   = "97"

    BGBRIGHTBLACK   = "100"
    BGBRIGHTRED     = "101"
    BGBRIGHTGREEN   = "102"
    BGBRIGHTYELLOW  = "103"
    BGBRIGHTBLUE    = "104"
    BGBRIGHTMAGENTA = "105"
    BGBRIGHTCYAN    = "106"
    BGBRIGHTWHITE   = "107"

def _cprint(fg: _EAnsiColours, bg: _EAnsiColours, text: str):
    print(f"\033[{fg.value};{bg.value}m{text}\033[{_EAnsiColours.FGWHITE.value};{_EAnsiColours.BGBLACK.value}m")

def printred(text: str):
    _cprint(_EAnsiColours.FGRED, _EAnsiColours.BGBLACK, text)

def printgreen(text: str):
    _cprint(_EAnsiColours.FGGREEN, _EAnsiColours.BGBLACK, text)



# Core Python.
import sys

# Local.
from .docker import DockerImage

# Get command line inputs.
if len(sys.argv) == 5:
    if (sys.argv[1] == "docker") and (sys.argv[2] == "image") and (sys.argv[3] == "save"):
        image_path = sys.argv[4]
        image = DockerImage(image_path)
        image.pull()
        image.save()

else:
    print("Command does not exist!")



# Core Python.
import json
import subprocess

class DockerImage:
    def __init__(self, image_path: str = ""):
        """Constructor"""
        self.image_path: str = image_path
    #==========================================================================
    # Internal, alphabetic.
    def _get_details(self):
        """Get image details."""
        cmd_parts = ["docker", "image", "inspect", self.image_path]
        proc = subprocess.run(
            cmd_parts,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE
        )
        self.details = json.loads(proc.stdout)
    def get_id(self):
        """Get image ID"""
        self._get_details()
        return self.details[0]["Id"]
    #==========================================================================
    # External, alphabetic.
    def set_path(self, image_path: str):
        """Set the image path"""
        self.image_path = image_path
    def pull(self):
        """Pull image to local machine"""
        print(f"Pulling image {self.image_path}", end="", flush=True)
        cmd_parts = [
            "docker",
            "image",
            "pull",
            self.image_path
            ]
        proc = subprocess.run(
            cmd_parts,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE
        )
        print("done!")
    def archive_name_to_image_path(self, archive_name: str):
        """Work out image path from archive name"""
        pass
    def get_archive_name(self):
        """Work out archive name from image path"""
        image_archive_name = self.image_path.replace("-", "_")
        image_archive_name = image_archive_name.replace("/", "__")
        image_archive_name = image_archive_name.replace(":", "___")
        return image_archive_name
    def save(self):
        """Save image to tar file"""
        print(f"Saving image {self.image_path} to archive file...", end="", flush=True)
        cmd_parts = [
            "docker",
            "image",
            "save",
            "--output-string",
            self.get_archive_name(),
            self.image_path
        ]
        proc = subprocess.run(
            cmd_parts,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE
        )
        print(f"done!")